// hlavickove subory *******************************************************
#include "includes.h"
// vlastny .h subor
#include "interface.h"

// uplynuty cas medzi aktualnym a povodnym casovym okamihom (s)
GLfloat dTime;

// instancia triedy CTexture - praca s texturami
CTexture texture;
// ciselne oznacenia textur - nie su totozne s identifikatormi nahratych textur v CTexture!
// vyhradene oznacenia:
// 0 - standardna textura
// 1 - zameriavaci kriz
// <10;(MAX_TEX_NUM-1)> - ostatne textury
GLuint tid[MAX_TEX_NUM];

// pocet segmentov gule, valca, ihlana
int segs=18;
// pocet udajov / 3 = pocet trojuholnikov
/*
int data_num=108+                          // kocka
             (360/segs*18)*(180/segs)+     // gula
             (360/segs*18)+(360/segs*9)*2+ // valec
             (360/segs*9)+(360/segs*9)+    // ihlan
             18;                           // plocha
*/
// vrcholy, farba, normaly, texturovacie suradnice
// 108 + 3600 + 720 + 360 + 18 = 4806
float v_a[4806],c_a[4806],n_a[4806],t_a[4806];

// destruktor **************************************************************
CInterface::~CInterface()
{

}

// inicializacia OpenGL - nastavenie parametrov OpenGL sceny ***************
int CInterface::OGL_initialize(void)
{
 printf("OpenGL init...\n");

 GLenum err=glewInit();
 if (err!=GLEW_OK) { printf("GLEW error: %s\n",glewGetErrorString(err)); return(1); }
 printf("using GLEW %s\n",glewGetString(GLEW_VERSION));

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 glClearColor(0.4f,0.4f,0.4f,1.0f);
 // zasobnik hlbky bude vymazavany nastavenim na 1.0f
 glClearDepth(1.0f);
 // pomocny zasobnik bude vymazavany nastavenim na 0.0f
 glClearStencil(0.0f);

 glDepthFunc(GL_LESS);    // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // testovanie Z-vzdialenosti

 glDisable(GL_CULL_FACE); // poradie, v ktorom budu definovane vrcholy objektov
 glFrontFace(GL_CCW);     // reprezentovat privratene, resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // texturovanie je standardne zapnute
 glEnable(GL_TEXTURE_2D);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // nastavenie ciselnych oznaceni textur
 // vyhradene oznacenia:
 // 0 - standardna textura
 // 1 - zameriavaci kriz
 // <10;(MAX_TEX_NUM-1)> - ostatne textury
 loopi(MAX_TEX_NUM) tid[i]=i;

 // inicializacia Avatara
 avatar.init();

 // inicializacia textu - nahratie textury s fontom
 if (text.init("data/font/font.png")) {
  printf("Chyba inicializacie textu!\n"); return(1);
 }

 // velkost zameriavacieho kriza
 crosshairSize=50.0f;

 // nahratie standardnej textury
 if (texture.load2D(tid[0],"data/textures/default/empty.png",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia standardnej textury!\n"); return(1);
 }

 // nahratie textury zameriavacieho kriza
 if (texture.load2D(tid[1],"data/gfx/crosshair1.png",T_LINEAR | T_CLAMP)) {
  printf("Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }

 // Vertex Array Object
 glGenVertexArrays(1,&vao);
 glBindVertexArray(vao);

 // inicializacia pouzitia shaderov
 if (base_shader.init()) return(1);
 // nacitanie shaderov
 if (base_shader.load("shaders/base.vp","shaders/base.fp")) return(1);

 // priprava objektov - kocka, gula, valec, ihlan, plocha
 objects_prepare();

 base_shader.on();
 base_shader.bindattriblocation(0,"in_pos"); // vrcholy
 base_shader.bindattriblocation(1,"in_col"); // farby
 base_shader.bindattriblocation(2,"in_nor"); // normaly
 base_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 base_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 base_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 base_shader.off();

 return(0);
}

// nastartovanie casovaca **************************************************
// 1s=1000ms
// Uint32 SDL_GetTicks() vracia pocet ms od startu SDL,
// po 49.7 dnoch dojde k prekroceniu 32bit limitu a pocitadlo sa vynuluje
void CInterface::resettimer(void)
{
 // aktualny casovy okamih (ms)
 actTime=(GLuint) SDL_GetTicks();
 // povodny casovy okamih (ms)
 oldTime=actTime;
 // pocet uplynutych milisekund
 millis=actTime-oldTime;
 // pomocne premenne
 fpsdelay=0; fpserror=0;

 // uplynuty cas medzi aktualnym a povodnym casovym okamihom (s)
 dTime=((GLfloat) millis)/1000.0f;

 // uplynute ms
 mspos=0; mstotal=0;
 // pocet snimkov za sekundu, minimalny a maximalny pocet snimkov za sekundu
 actfps=0; minfps=0; maxfps=0;
}

// aktualny pocet fps, minimalny a maximalny pocet fps *********************
void CInterface::getfps(GLuint &ACT,GLuint &MIN,GLuint &MAX)
{
 // posledny prvok pola uplynutych ms
 mstotal=mshistory[MSHISTORY-1]; MIN=mstotal; MAX=mstotal;
 // ostatne prvky pola
 loopi(MSHISTORY-1) {
  mstotal+=mshistory[i];
  if (mshistory[i]>MIN) MIN=mshistory[i];
  if (mshistory[i]<MAX) MAX=mshistory[i];
 }
 ACT=(1000*MSHISTORY)/mstotal; MIN=1000/MIN; MAX=1000/MAX;
}

// aktualizacia casovaca ***************************************************
void CInterface::updatetimer(void)
{
 // aktualny casovy okamih (ms)
 actTime=(GLuint) SDL_GetTicks();
 // aktualny casovy okamih nemoze byt "starsi" ako povodny casovy okamih
 if (actTime<oldTime) actTime=oldTime;
 // pocet uplynutych milisekund
 millis=actTime-oldTime;
 // uprava poctu uplynutych milisekund
 fpsdelay=1000/MAXFPS-millis;
 if (fpsdelay<0) fpserror=0; // ak MAXFPS=200 => ak millis>=5
 else {
  fpserror+=1000%MAXFPS; // ak MAXFPS=200 => fpserror+=0
  if (fpserror>=MAXFPS) { fpserror-=MAXFPS; fpsdelay++; }
  if (fpsdelay>0) { // ak millis<5
   SDL_Delay(fpsdelay); // pauza
   actTime+=(GLuint) fpsdelay; // aktualny casovy okamih (ms)
   millis=actTime-oldTime; // pocet uplynutych milisekund
  }
 }

 // uplynuty cas medzi aktualnym a povodnym casovym okamihom (s)
 dTime=((GLfloat) millis)/1000.0f;
 oldTime=actTime;

 // aktualizacia pola uplynutych ms
 mshistory[mspos++]=(GLuint) max(1,min(1000,(int) millis));
 // po naplneni pola uplynutych ms sa vypocita actfps, minfps a maxfps
 if (mspos>=MSHISTORY) {
  mspos=0;
  // aktualny pocet fps, minimalny a maximalny pocet fps
  getfps(actfps,minfps,maxfps);
  // INFO: fps
  //printf("%u %u %u\n",actfps,minfps,maxfps);
 }
}

// casovo zavisla uprava objektov, resp. sceny *****************************
void CInterface::OGL_idle(void)
{

}

// vykreslenie OpenGL sceny ************************************************
void CInterface::OGL_draw(void)
{
 // vymazanie zasobnika farieb, hlbky a Stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

 // INFO: bez shaderov
/*
 // modelview matica
 glLoadMatrixf(avatar.get_mv_mat());
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[0]]);
 // vykreslenie objektov - kocka, gula, valec, ihlan, plocha
 objects_draw();
*/
 base_shader.on();
 // modelview matica
 base_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[0]]);
 // vykreslenie objektov - kocka, gula, valec, ihlan, plocha
 objects_draw();
 base_shader.off();

 // vykreslenie head-up display
 drawhud();
}

// vykreslenie head-up display *********************************************
void CInterface::drawhud(void)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT | GL_DEPTH_BUFFER_BIT);

 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glEnable(GL_BLEND);       // zapnutie priesvitnosti

 glMatrixMode(GL_PROJECTION); // projekcna matica
 glPushMatrix();              // ulozenie
 glLoadIdentity();            // reset

 // nastavenie pravouhleho premietania - obrati poradie CCW na CW
 glOrtho(0.0f,(GLfloat) scr_w,(GLfloat) scr_h,0.0f,-1.0f,1.0f);

 glMatrixMode(GL_MODELVIEW); // modelview matica
 glPushMatrix();             // ulozenie
 glLoadIdentity();           // reset

 // funkcia pre vypocet priesvitnosti
 glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA);

 // vykreslenie menu -------------------------------------------------------

 // farba menu
 glColor4f(1.0f,1.0f,1.0f,0.5f);
 // stvoruholnik
 glBegin(GL_QUADS); // poradie vrcholov je CW pretoze pre vykreslenie je pouzite glOrtho()
  glVertex3f(5.0f,5.0f,0.0f);
  glVertex3f(5.0f,(GLfloat) (scr_h-5),0.0f);
  glVertex3f(145.f,(GLfloat) (scr_h-5),0.0f);
  glVertex3f(145.0f,5.0f,0.0f);
 glEnd();

 // vykreslenie textu ------------------------------------------------------

 // farba textu
 glColor4f(0.8f,0.8f,0.0f,1.0f);
 // pripojenie textury s fontom
 glBindTexture(GL_TEXTURE_2D,text.tex_id);

 glPushMatrix(); // ulozenie modelview matice
 // velkost textu
 glScalef(0.5f,0.5f,1.0f);
 // vypis poctu fps
 text.drawf(15.0f,15.0f,"%u",actfps);
 glPopMatrix(); // obnova modelview matice

 glPushMatrix(); // ulozenie modelview matice
 // velkost textu
 glScalef(0.5f,0.5f,1.0f);
 // poloha Avatara
 text.drawf(15.0f,45.0f,"%.1f %.1f %.1f",avatar.actpos.x,avatar.actpos.y,avatar.actpos.z);
 glPopMatrix(); // obnova modelview matice

 // vykreslenie zameriavacieho kriza ---------------------------------------

 // funkcia pre vypocet priesvitnosti
 glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR);

 // farba zameriavacieho kriza
 glColor4f(0.5f,0.5f,0.5f,1.0f);
 // pripojenie textury zameriavacieho kriza
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[1]]);

 glPushMatrix(); // ulozenie modelview matice
 // zameriavaci kriz na stred okna
 glTranslatef((GLfloat) scr_w2-crosshairSize/2.0f,
              (GLfloat) scr_h2-crosshairSize/2.0f,0.0f);
 // stvoruholnik
 glBegin(GL_QUADS); // poradie vrcholov je CW pretoze pre vykreslenie je pouzite glOrtho()
  glTexCoord2f(0.0f,0.0f); glVertex3f(0.0f,0.0f,0.0f);
  glTexCoord2f(0.0f,1.0f); glVertex3f(0.0f,crosshairSize,0.0f);
  glTexCoord2f(1.0f,1.0f); glVertex3f(crosshairSize,crosshairSize,0.0f);
  glTexCoord2f(1.0f,0.0f); glVertex3f(crosshairSize,0.0f,0.0f);
 glEnd();
 glPopMatrix(); // obnova modelview matice

 // nastavenie standardnej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 glPopMatrix(); // obnova modelview matice
 glMatrixMode(GL_PROJECTION);
 glPopMatrix(); // obnova projection matice

 // aktualna je vzdy modelview matica
 glMatrixMode(GL_MODELVIEW);

 glDisable(GL_BLEND); // vypnutie priesvitnosti
 glPopAttrib();       // vyber ulozenych atributov - LIGHTING, DEPTH_BUFFER
}

// priprava objektov - kocka, gula, valec, ihlan, plocha *******************
void CInterface::objects_prepare(void)
{
 // pomocne premenne
 int q=0,w=0,e=0;
 float alfa[2],beta[2];
 float x[4],y[2],z[4];

 e=108; // pocet udajov patriacich kocke

 // kocka ------------------------------------------------------------------
 float tmp[]={-4.0f, 0.0f, 4.0f,-2.0f, 0.0f, 4.0f,-2.0f, 2.0f, 4.0f,
              -4.0f, 0.0f, 4.0f,-2.0f, 2.0f, 4.0f,-4.0f, 2.0f, 4.0f,
              -2.0f, 0.0f, 4.0f,-2.0f, 0.0f, 2.0f,-2.0f, 2.0f, 2.0f,
              -2.0f, 0.0f, 4.0f,-2.0f, 2.0f, 2.0f,-2.0f, 2.0f, 4.0f,
              -2.0f, 0.0f, 2.0f,-4.0f, 0.0f, 2.0f,-4.0f, 2.0f, 2.0f,
              -2.0f, 0.0f, 2.0f,-4.0f, 2.0f, 2.0f,-2.0f, 2.0f, 2.0f,
              -4.0f, 0.0f, 2.0f,-4.0f, 0.0f, 4.0f,-4.0f, 2.0f, 4.0f,
              -4.0f, 0.0f, 2.0f,-4.0f, 2.0f, 4.0f,-4.0f, 2.0f, 2.0f,
              -4.0f, 2.0f, 4.0f,-2.0f, 2.0f, 4.0f,-2.0f, 2.0f, 2.0f,
              -4.0f, 2.0f, 4.0f,-2.0f, 2.0f, 2.0f,-4.0f, 2.0f, 2.0f,
              -4.0f, 0.0f, 2.0f,-2.0f, 0.0f, 2.0f,-2.0f, 0.0f, 4.0f,
              -4.0f, 0.0f, 2.0f,-2.0f, 0.0f, 4.0f,-4.0f, 0.0f, 4.0f};
 loopi(e) v_a[i]=tmp[i];
 // farby
 for (int i=0;i<e;i+=3) { c_a[i]=0.5f; c_a[i+1]=0.0f; c_a[i+2]=0.0f; }
 // normaly
 float tmp1[]={ 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f,
                1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
                1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
                0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f,
                0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f,
               -1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,
               -1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,
                0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f,
                0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f,
                0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f,
                0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f};
 loopi(e) n_a[i]=tmp1[i];
 // texturovacie suradnice
 float tmp2[]={ 0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f};
 loopi(72) t_a[i]=tmp2[i];

 // gula -------------------------------------------------------------------
 loopj(180) {
  if (j%segs) continue;
  loopi(360) {
   if (i%segs) continue;

   alfa[0]=DEG2RAD((float) i);        beta[0]=DEG2RAD((float) j);
   alfa[1]=DEG2RAD((float) (i+segs)); beta[1]=DEG2RAD((float) (j+segs));

   x[0]=3.0f+cos(alfa[0])*sin(beta[0]); x[1]=3.0f+cos(alfa[1])*sin(beta[0]);
   x[2]=3.0f+cos(alfa[1])*sin(beta[1]); x[3]=3.0f+cos(alfa[0])*sin(beta[1]);
   y[0]=1.0f+cos(beta[0]); y[1]=1.0f+cos(beta[1]);
   z[0]=3.0f+sin(alfa[0])*sin(beta[0]); z[1]=3.0f+sin(alfa[1])*sin(beta[0]);
   z[2]=3.0f+sin(alfa[1])*sin(beta[1]); z[3]=3.0f+sin(alfa[0])*sin(beta[1]);

   v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
   v_a[e+q+3]=x[1]; v_a[e+q+4]=y[0]; v_a[e+q+5]=z[1]; // 2. vrchol
   v_a[e+q+6]=x[2]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[2]; // 3. vrchol
   c_a[e+q+0]=0.0f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
   c_a[e+q+3]=0.0f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
   c_a[e+q+6]=0.0f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
   n_a[e+q+0]=1.0f; n_a[e+q+1]=0.0f; n_a[e+q+2]=0.0f; // 1. vrchol
   n_a[e+q+3]=1.0f; n_a[e+q+4]=0.0f; n_a[e+q+5]=0.0f; // 2. vrchol
   n_a[e+q+6]=1.0f; n_a[e+q+7]=0.0f; n_a[e+q+8]=0.0f; // 3. vrchol
   t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=(float) j/90.0f;        // 1. vrchol
   t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=(float) j/90.0f;        // 2. vrchol
   t_a[72+w+4]=(float) (i+segs)/90.0f; t_a[72+w+5]=(float) (j+segs)/90.0f; // 3. vrchol
   q+=9; w+=6;

   v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
   v_a[e+q+3]=x[2]; v_a[e+q+4]=y[1]; v_a[e+q+5]=z[2]; // 2. vrchol
   v_a[e+q+6]=x[3]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[3]; // 3. vrchol
   c_a[e+q+0]=0.0f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
   c_a[e+q+3]=0.0f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
   c_a[e+q+6]=0.0f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
   n_a[e+q+0]=1.0f; n_a[e+q+1]=0.0f; n_a[e+q+2]=0.0f; // 1. vrchol
   n_a[e+q+3]=1.0f; n_a[e+q+4]=0.0f; n_a[e+q+5]=0.0f; // 2. vrchol
   n_a[e+q+6]=1.0f; n_a[e+q+7]=0.0f; n_a[e+q+8]=0.0f; // 3. vrchol
   t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=(float) j/90.0f;        // 1. vrchol
   t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=(float) (j+segs)/90.0f; // 2. vrchol
   t_a[72+w+4]=(float) i/90.0f;        t_a[72+w+5]=(float) (j+segs)/90.0f; // 3. vrchol
   q+=9; w+=6;
  }
 }

 // valec -------------------------------------------------------------------
 loopi(360) {
  if (i%segs) continue;

  alfa[0]=DEG2RAD((float) i);
  alfa[1]=DEG2RAD((float) (i+segs));

  x[0]=3.0f+cos(alfa[0]); x[1]=3.0f+cos(alfa[1]);
  y[0]=0.0f; y[1]=2.0f;
  z[0]=-3.0f-sin(alfa[0]); z[1]=-3.0f-sin(alfa[1]);

  v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
  v_a[e+q+3]=x[1]; v_a[e+q+4]=y[0]; v_a[e+q+5]=z[1]; // 2. vrchol
  v_a[e+q+6]=x[1]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[1]; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  n_a[e+q+0]=1.0f; n_a[e+q+1]=0.0f; n_a[e+q+2]=0.0f; // 1. vrchol
  n_a[e+q+3]=1.0f; n_a[e+q+4]=0.0f; n_a[e+q+5]=0.0f; // 2. vrchol
  n_a[e+q+6]=1.0f; n_a[e+q+7]=0.0f; n_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=(float) (i+segs)/90.0f; t_a[72+w+5]=1.0f; // 3. vrchol
  q+=9; w+=6;

  v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
  v_a[e+q+3]=x[1]; v_a[e+q+4]=y[1]; v_a[e+q+5]=z[1]; // 2. vrchol
  v_a[e+q+6]=x[0]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[0]; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  n_a[e+q+0]=1.0f; n_a[e+q+1]=0.0f; n_a[e+q+2]=0.0f; // 1. vrchol
  n_a[e+q+3]=1.0f; n_a[e+q+4]=0.0f; n_a[e+q+5]=0.0f; // 2. vrchol
  n_a[e+q+6]=1.0f; n_a[e+q+7]=0.0f; n_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=1.0f; // 2. vrchol
  t_a[72+w+4]=(float) i/90.0f;        t_a[72+w+5]=1.0f; // 3. vrchol
  q+=9; w+=6;

  v_a[e+q+0]=x[0]; v_a[e+q+1]=2.0f; v_a[e+q+2]=z[0];  // 1. vrchol
  v_a[e+q+3]=x[1]; v_a[e+q+4]=2.0f; v_a[e+q+5]=z[1];  // 2. vrchol
  v_a[e+q+6]=3.0f; v_a[e+q+7]=2.0f; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  n_a[e+q+0]=0.0f; n_a[e+q+1]=1.0f; n_a[e+q+2]=0.0f; // 1. vrchol
  n_a[e+q+3]=0.0f; n_a[e+q+4]=1.0f; n_a[e+q+5]=0.0f; // 2. vrchol
  n_a[e+q+6]=0.0f; n_a[e+q+7]=1.0f; n_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=0.1f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=0.1f; t_a[72+w+5]=1.0f; // 3. vrchol
  q+=9; w+=6;

  v_a[e+q+0]=x[1]; v_a[e+q+1]=0.0f; v_a[e+q+2]=z[1];  // 1. vrchol
  v_a[e+q+3]=x[0]; v_a[e+q+4]=0.0f; v_a[e+q+5]=z[0];  // 2. vrchol
  v_a[e+q+6]=3.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  n_a[e+q+0]=0.0f; n_a[e+q+1]=-1.0f; n_a[e+q+2]=0.0f; // 1. vrchol
  n_a[e+q+3]=0.0f; n_a[e+q+4]=-1.0f; n_a[e+q+5]=0.0f; // 2. vrchol
  n_a[e+q+6]=0.0f; n_a[e+q+7]=-1.0f; n_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=0.1f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=0.1f; t_a[72+w+5]=1.0f; // 3. vrchol
  q+=9; w+=6;
 }

 // ihlan ------------------------------------------------------------------
 loopi(360) {
  if (i%segs) continue;

  alfa[0]=DEG2RAD((float) i);
  alfa[1]=DEG2RAD((float) (i+segs));

  x[0]=-3.0f+cos(alfa[0]); x[1]=-3.0f+cos(alfa[1]);
  y[0]=0.0f; y[1]=2.0f;
  z[0]=-3.0f-sin(alfa[0]); z[1]=-3.0f-sin(alfa[1]);

  v_a[e+q+0]=x[0];  v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0];  // 1. vrchol
  v_a[e+q+3]=x[1];  v_a[e+q+4]=y[0]; v_a[e+q+5]=z[1];  // 2. vrchol
  v_a[e+q+6]=-3.0f; v_a[e+q+7]=y[1]; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.5f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
  c_a[e+q+3]=0.5f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
  c_a[e+q+6]=0.5f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
  n_a[e+q+0]=1.0f; n_a[e+q+1]=0.0f; n_a[e+q+2]=0.0f; // 1. vrchol
  n_a[e+q+3]=1.0f; n_a[e+q+4]=0.0f; n_a[e+q+5]=0.0f; // 2. vrchol
  n_a[e+q+6]=1.0f; n_a[e+q+7]=0.0f; n_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=0.1f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=0.1f; t_a[72+w+5]=1.0f; // 3. vrchol
  q+=9; w+=6;

  v_a[e+q+0]=x[1];  v_a[e+q+1]=0.0f; v_a[e+q+2]=z[1];  // 1. vrchol
  v_a[e+q+3]=x[0];  v_a[e+q+4]=0.0f; v_a[e+q+5]=z[0];  // 2. vrchol
  v_a[e+q+6]=-3.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.5f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
  c_a[e+q+3]=0.5f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
  c_a[e+q+6]=0.5f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
  n_a[e+q+0]=0.0f; n_a[e+q+1]=-1.0f; n_a[e+q+2]=0.0f; // 1. vrchol
  n_a[e+q+3]=0.0f; n_a[e+q+4]=-1.0f; n_a[e+q+5]=0.0f; // 2. vrchol
  n_a[e+q+6]=0.0f; n_a[e+q+7]=-1.0f; n_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=0.1f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=0.1f; t_a[72+w+5]=1.0f; // 3. vrchol
  q+=9; w+=6;
 }

 // plocha -----------------------------------------------------------------
 v_a[e+q+0]=-5.0f; v_a[e+q+1]=0.0f; v_a[e+q+2]= 5.0f; // 1. vrchol
 v_a[e+q+3]= 5.0f; v_a[e+q+4]=0.0f; v_a[e+q+5]= 5.0f; // 2. vrchol
 v_a[e+q+6]= 5.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-5.0f; // 3. vrchol
 c_a[e+q+0]=0.2f; c_a[e+q+1]=0.2f; c_a[e+q+2]=0.2f; // 1. vrchol
 c_a[e+q+3]=0.2f; c_a[e+q+4]=0.2f; c_a[e+q+5]=0.2f; // 2. vrchol
 c_a[e+q+6]=0.2f; c_a[e+q+7]=0.2f; c_a[e+q+8]=0.2f; // 3. vrchol
 n_a[e+q+0]=0.0f; n_a[e+q+1]=1.0f; n_a[e+q+2]=0.0f; // 1. vrchol
 n_a[e+q+3]=0.0f; n_a[e+q+4]=1.0f; n_a[e+q+5]=0.0f; // 2. vrchol
 n_a[e+q+6]=0.0f; n_a[e+q+7]=1.0f; n_a[e+q+8]=0.0f; // 3. vrchol
 t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
 t_a[72+w+2]=1.0f; t_a[72+w+3]=0.0f; // 2. vrchol
 t_a[72+w+4]=1.0f; t_a[72+w+5]=1.0f; // 3. vrchol
 q+=9; w+=6;

 v_a[e+q+0]=-5.0f; v_a[e+q+1]=0.0f; v_a[e+q+2]= 5.0f; // 1. vrchol
 v_a[e+q+3]= 5.0f; v_a[e+q+4]=0.0f; v_a[e+q+5]=-5.0f; // 2. vrchol
 v_a[e+q+6]=-5.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-5.0f; // 3. vrchol
 c_a[e+q+0]=0.2f; c_a[e+q+1]=0.2f; c_a[e+q+2]=0.2f; // 1. vrchol
 c_a[e+q+3]=0.2f; c_a[e+q+4]=0.2f; c_a[e+q+5]=0.2f; // 2. vrchol
 c_a[e+q+6]=0.2f; c_a[e+q+7]=0.2f; c_a[e+q+8]=0.2f; // 3. vrchol
 n_a[e+q+0]=0.0f; n_a[e+q+1]=1.0f; n_a[e+q+2]=0.0f; // 1. vrchol
 n_a[e+q+3]=0.0f; n_a[e+q+4]=1.0f; n_a[e+q+5]=0.0f; // 2. vrchol
 n_a[e+q+6]=0.0f; n_a[e+q+7]=1.0f; n_a[e+q+8]=0.0f; // 3. vrchol
 t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
 t_a[72+w+2]=1.0f; t_a[72+w+3]=1.0f; // 2. vrchol
 t_a[72+w+4]=0.0f; t_a[72+w+5]=1.0f; // 3. vrchol
 q+=9; w+=6;

 o.tris_num=36+q/3; // kocka+gula+valec+ihlan+plocha
 printf("pocet trojuholnikov: %i\n",o.tris_num);

 o.buffers_prepare();

 o.vertices->load(sizeof(v_a),v_a);
 o.colors->load(sizeof(c_a),c_a);
 o.normals->load(sizeof(n_a),n_a);
 o.texcoords->load(sizeof(t_a),t_a);

#ifdef OGL20
 // vrcholy
 glEnableVertexAttribArray(0);                     // povolenie pola atributov
 o.vertices->bind();                               // pripojenie buffra
 glVertexAttribPointer(0,3,GL_FLOAT,GL_FALSE,0,0); // ukazovatel na udaje
 // farby
 glEnableVertexAttribArray(1);
 o.colors->bind();
 glVertexAttribPointer(1,3,GL_FLOAT,GL_FALSE,0,0);
 // normaly
 glEnableVertexAttribArray(2);
 o.normals->bind();
 glVertexAttribPointer(2,3,GL_FLOAT,GL_FALSE,0,0);
 // texturovacie suradnice
 glEnableVertexAttribArray(8);
 o.texcoords->bind();
 glVertexAttribPointer(8,2,GL_FLOAT,GL_FALSE,0,0);
#else
 // vrcholy
 glEnableVertexAttribArrayARB(0);
 o.vertices->bind();
 glVertexAttribPointerARB(0,3,GL_FLOAT,GL_FALSE,0,0);
 // farby
 glEnableVertexAttribArrayARB(1);
 o.colors->bind();
 glVertexAttribPointerARB(1,3,GL_FLOAT,GL_FALSE,0,0);
 // normaly
 glEnableVertexAttribArrayARB(2);
 o.normals->bind();
 glVertexAttribPointerARB(2,3,GL_FLOAT,GL_FALSE,0,0);
 // texturovacie suradnice
 glEnableVertexAttribArrayARB(8);
 o.texcoords->bind();
 glVertexAttribPointerARB(8,2,GL_FLOAT,GL_FALSE,0,0);
#endif
 // INFO: bez shaderov
/*
 glEnableClientState(GL_VERTEX_ARRAY);
 glVertexPointer(3,GL_FLOAT,0,v_a);
 glEnableClientState(GL_COLOR_ARRAY);
 glColorPointer(3,GL_FLOAT,0,c_a);
*/
}

// vykreslanie objektov - kocka, gula, valec, ihlan, plocha ****************
void CInterface::objects_draw(void)
{
 glDrawArrays(GL_TRIANGLES,0,o.tris_num);
}
