#ifndef _INTERFACE_H_
#define _INTERFACE_H_

// pole uplynutych ms
#define MSHISTORY 20

// deklaracia tried pre pracu s jednotlivymi modulmi
#include "common/avatar.h"
#include "common/text.h"

// OpenGL zasobnik - buffer
class GLbuf
{
public:
 // konstruktor
 GLbuf(GLenum T=GL_ARRAY_BUFFER) {
#ifdef OGL20
  glGenBuffers(1,&buf); // vygenerovanie 1 zasobnika
#else
  glGenBuffersARB(1,&buf);
#endif
  target=T;             // ciel, standardne GL_ARRAY_BUFFER
 }
 // destruktor
 ~GLbuf() {
#ifdef OGL20
  glDeleteBuffers(1,&buf);
#else
  glDeleteBuffersARB(1,&buf);
#endif
 }
 // nahratie udajov
 void load(size_t SIZE,void *DATA=NULL,GLenum USAGE=GL_STATIC_DRAW) {
#ifdef OGL20
  glBindBuffer(target,buf);             // zasobnik, s ktorym sa bude pracovat
  glBufferData(target,SIZE,DATA,USAGE); // nahratie udajov do zasobnika (moze byt aj NULL)
#else
  glBindBufferARB(target,buf);
  glBufferDataARB(target,SIZE,DATA,USAGE);
#endif
 }
 // zasobnik, s ktorym sa bude pracovat
 void bind(void) {
#ifdef OGL20
  glBindBuffer(target,buf);
#else
  glBindBufferARB(target,buf);
#endif
 }
private:
 GLuint buf;    // identifikator
 GLenum target; // ciel
};

// OpenGL objekt - kocka, gula, valec, ihlan, plocha
class GLobj
{
public:
 // konstruktor
 GLobj() { vertices=NULL; colors=NULL; normals=NULL; texcoords=NULL; tris_num=0; }
 // destruktor
 ~GLobj() { DELETEP(vertices); DELETEP(colors); DELETEP(normals); DELETEP(texcoords); }

 GLbuf *vertices;  // vrcholy
 GLbuf *colors;    // farby
 GLbuf *normals;   // normaly
 GLbuf *texcoords; // texturovacie suradnice

 GLuint tris_num; // pocet trojuholnikov

 void buffers_prepare(void) {
  vertices=new GLbuf;  // vrcholy
  colors=new GLbuf;    // farby
  normals=new GLbuf;   // normaly
  texcoords=new GLbuf; // texturovacie suradnice
 }
};

// trieda premennych a funkcii pre pracu s OpenGL
class CInterface {
public:
 CInterface() {};  // konstruktor
 ~CInterface(); // destruktor

 CAvatar avatar; // instancia triedy CAvatar - praca s kamerou
 CText text;     // instancia triedy CText - praca s textom

 // inicializacia OpenGL - nastavenie parametrov OpenGL sceny
 int OGL_initialize(void);

 void resettimer(void);  // nastartovanie casovaca
 void updatetimer(void); // aktualizacia casovaca

 void OGL_idle(void); // casovo zavisla uprava objektov, resp. sceny
 void OGL_draw(void); // vykreslenie OpenGL sceny
private:
 // aktualny a povodny casovy okamih (ms), pocet uplynutych milisekund
 GLuint actTime,oldTime,millis;
 // pomocne premenne
 GLint fpsdelay,fpserror;
 // uplynute ms
 GLuint mspos,mstotal,mshistory[MSHISTORY];
 // pocet snimkov za sekundu, minimalny a maximalny pocet snimkov za sekundu
 GLuint actfps,minfps,maxfps;

 // velkost zameriavacieho kriza
 GLfloat crosshairSize;

 GLuint vao; // Vertex Array Object

 GLobj o; // kocka, gula, valec, ihlan, plocha

 // aktualny pocet fps, minimalny a maximalny pocet fps
 void getfps(GLuint &ACT,GLuint &MIN,GLuint &MAX);

 // vykreslenie head-up display
 void drawhud(void);

 // priprava objektov - kocka, gula, valec, ihlan, plocha
 void objects_prepare(void);
 // vykreslenie objektov - kocka, gula, valec, ihlan, plocha
 void objects_draw(void);
};

#endif
